unit can.error;

interface

uses
  ctypes;

(* SPDX-License-Identifier: ((GPL-2.0-only WITH Linux-syscall-note) OR BSD-3-Clause) *)
(*
 * linux/can/error.h
 *
 * Definitions of the CAN error messages to be filtered and passed to the user.
 *
 * Author: Oliver Hartkopp <oliver.hartkopp@volkswagen.de>
 * Copyright (c) 2002-2007 Volkswagen Group Electronic Research
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Volkswagen nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * Alternatively, provided that this notice is retained in full, this
 * software may be distributed under the terms of the GNU General
 * Public License ('GPL') version 2, in which case the provisions of the
 * GPL apply INSTEAD OF those given above.
 *
 * The provided data structures and external interfaces from this code
 * are not restricted to be used by modules with a GPL compatible license.
 *
 *  SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * 'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING  ANY WAY  OF THE USE
 * OF  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *)

const
  CAN_ERR_DLC = 8;	                           (* dlc for error message frames        *)

  (* error class (mask) in can_id *)
  CAN_ERR_TX_TIMEOUT: cuint32    = $00000001;  (* TX timeout (by netdevice driver)    *)
  CAN_ERR_LOSTARB:    cuint32    = $00000002;  (* lost arbitration    / data[0]       *)
  CAN_ERR_CRTL:       cuint32    = $00000004;  (* controller problems / data[1]       *)
  CAN_ERR_PROT:       cuint32    = $00000008;  (* protocol violations / data[2..3]    *)
  CAN_ERR_TRX:        cuint32    = $00000010;  (* transceiver status  / data[4]       *)
  CAN_ERR_ACK:        cuint32    = $00000020;  (* received no ACK on transmission     *)
  CAN_ERR_BUSOFF:     cuint32    = $00000040;  (* bus off                             *)
  CAN_ERR_BUSERROR:   cuint32    = $00000080;  (* bus error (may flood!)              *)
  CAN_ERR_RESTARTED:  cuint32    = $00000100;  (* controller restarted                *)

  (* arbitration lost in bit ... / data[0] *)
  CAN_ERR_LOSTARB_UNSPEC         = $00;        (* unspecified                         *)
                                               (* else bit number in bitstream        *)

  (* error status of CAN-controller / data[1] *)
  CAN_ERR_CRTL_UNSPEC            = $00;        (* unspecified                         *)
  CAN_ERR_CRTL_RX_OVERFLOW       = $01;        (* RX buffer overflow                  *)
  CAN_ERR_CRTL_TX_OVERFLOW       = $02;        (* TX buffer overflow                  *)
  CAN_ERR_CRTL_RX_WARNING        = $04;        (* reached warning level for RX errors *)
  CAN_ERR_CRTL_TX_WARNING        = $08;        (* reached warning level for TX errors *)
  CAN_ERR_CRTL_RX_PASSIVE        = $10;        (* reached error passive status RX     *)
  CAN_ERR_CRTL_TX_PASSIVE        = $20;        (* reached error passive status TX     *)
	                                             (* (at least one error counter exceeds *)
				                                       (* the protocol-defined level of 127)  *)
  CAN_ERR_CRTL_ACTIVE            = $40;        (* recovered to error active state     *)

  (* error in CAN protocol (type) / data[2] *)
  CAN_ERR_PROT_UNSPEC            = $00;        (* unspecified                         *)
  CAN_ERR_PROT_BIT               = $01;        (* single bit error                    *)
  CAN_ERR_PROT_FORM              = $02;	       (* frame format error                  *)
  CAN_ERR_PROT_STUFF             = $04;	       (* bit stuffing error                  *)
  CAN_ERR_PROT_BIT0              = $08;	       (* unable to send dominant bit         *)
  CAN_ERR_PROT_BIT1              = $10;	       (* unable to send recessive bit        *)
  CAN_ERR_PROT_OVERLOAD          = $20;	       (* bus overload                        *)
  CAN_ERR_PROT_ACTIVE            = $40;	       (* active error announcement           *)
  CAN_ERR_PROT_TX                = $80;        (* error occurred on transmission      *)

  (* error in CAN protocol (location) / data[3] *)
  CAN_ERR_PROT_LOC_UNSPEC        = $00;        (* unspecified                         *)
  CAN_ERR_PROT_LOC_SOF           = $03;        (* start of frame                      *)
  CAN_ERR_PROT_LOC_ID28_21       = $02;        (* ID bits 28 - 21 (SFF: 10 - 3)       *)
  CAN_ERR_PROT_LOC_ID20_18       = $06;        (* ID bits 20 - 18 (SFF: 2 - 0 )       *)
  CAN_ERR_PROT_LOC_SRTR          = $04;        (* substitute RTR (SFF: RTR)           *)
  CAN_ERR_PROT_LOC_IDE           = $05;        (* identifier extension                *)
  CAN_ERR_PROT_LOC_ID17_13       = $07;        (* ID bits 17-13                       *)
  CAN_ERR_PROT_LOC_ID12_05       = $0F;        (* ID bits 12-5                        *)
  CAN_ERR_PROT_LOC_ID04_00       = $0E;        (* ID bits 4-0                         *)
  CAN_ERR_PROT_LOC_RTR           = $0C;        (* RTR                                 *)
  CAN_ERR_PROT_LOC_RES1          = $0D;        (* reserved bit 1                      *)
  CAN_ERR_PROT_LOC_RES0          = $09;        (* reserved bit 0                      *)
  CAN_ERR_PROT_LOC_DLC           = $0B;        (* data length code                    *)
  CAN_ERR_PROT_LOC_DATA          = $0A;        (* data section                        *)
  CAN_ERR_PROT_LOC_CRC_SEQ       = $08;        (* CRC sequence                        *)
  CAN_ERR_PROT_LOC_CRC_DEL       = $18;        (* CRC delimiter                       *)
  CAN_ERR_PROT_LOC_ACK           = $19;        (* ACK slot                            *)
  CAN_ERR_PROT_LOC_ACK_DEL       = $1B;        (* ACK delimiter                       *)
  CAN_ERR_PROT_LOC_EOF           = $1A;        (* end of frame                        *)
  CAN_ERR_PROT_LOC_INTERM        = $12;        (* intermission                        *)

  (* error status of CAN-transceiver / data[4] *)
                                               (* CANH CANL *)
  CAN_ERR_TRX_UNSPEC             = $00;        (* 0000 0000 *)
  CAN_ERR_TRX_CANH_NO_WIRE       = $04;        (* 0000 0100 *)
  CAN_ERR_TRX_CANH_SHORT_TO_BAT  = $05;        (* 0000 0101 *)
  CAN_ERR_TRX_CANH_SHORT_TO_VCC  = $06;        (* 0000 0110 *)
  CAN_ERR_TRX_CANH_SHORT_TO_GND  = $07;        (* 0000 0111 *)
  CAN_ERR_TRX_CANL_NO_WIRE       = $40;        (* 0100 0000 *)
  CAN_ERR_TRX_CANL_SHORT_TO_BAT  = $50;        (* 0101 0000 *)
  CAN_ERR_TRX_CANL_SHORT_TO_VCC  = $60;        (* 0110 0000 *)
  CAN_ERR_TRX_CANL_SHORT_TO_GND  = $70;        (* 0111 0000 *)
  CAN_ERR_TRX_CANL_SHORT_TO_CANH = $80;        (* 1000 0000 *)

  (* controller specific additional information / data[5..7] *)

implementation

end.

