//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This is a SocketCAN BCM cyclic demo created by Zeljko Avramovic             //
//  FPC modified LGPL licence: https://wiki.freepascal.org/FPC_modified_LGPL    //
//                                                                              //
//  Virtual CAN adapter vcan0 is hard coded and you can bring it up like this:  //
//  sudo modprobe vcan                                                          //
//  sudo ip link add dev vcan0 type vcan                                        //
//  sudo ip link set vcan0 mtu 72            # needed for CAN FD                //
//  sudo ip link set vcan0 up                                                   //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

program hlcancyclic; // higher level pascalized version of cancyclic demo

{$mode delphi} {$H+}

uses
  cthreads, sysutils, sockets, crt, can, can.hl, can.bcm;

type
  TBcmMsg = record
    Head: TBcmMsgHeadWithoutFrames;  // we shall use more then 1 frame so TBcmMsgHead is not enough
    Frame: array[0..2] of TCanFrame; // and we add here space for our 3 frames
  end;

procedure perror(const S : string);
begin
  WriteLn(S, ', SocketError = ', SocketError);
end;

var
  i:            byte;
  CanHandle:    TCanHandle;
  CanAddr:      TCanSockAddr;
  CanInterface: TCanIfreq;
  Msg:          TBcmMsg;
begin
  try
    Writeln('CAN Sockets BCM Cyclic Demo');
    CanHandle := CanSocket(SOCK_DGRAM, CAN_BCM);
    if CanHandle < 0 then
    begin
      perror('Socket');
      Exit;
    end;

    CanInterface.Name := 'vcan0' + #0;
    if CanIOCtl(CanHandle, SIOCGIFINDEX, CanInterface) < 0 then
    begin
      perror('IOctl');
      Exit;
    end;

    CanAddr.Clear;
    CanAddr.CanFamily  := AF_CAN;
    CanAddr.CanIfIndex := CanInterface.IfIndex;

    if CanConnect(CanHandle, CanAddr) < 0 then
    begin
      perror('Connect');
      Exit;
    end;

    Msg.Head.OpCode        := TX_SETUP;
    Msg.Head.ID.Value      := $567; // if TX_CP_CAN_ID flag is set then all messages shall have this CAN ID
    Msg.Head.Flags         := SETTIMER OR STARTTIMER OR TX_ANNOUNCE {OR TX_CP_CAN_ID} {OR TX_RESET_MULTI_IDX};
    Msg.Head.NFrames       := 3;
    Msg.Head.Count         := 0; // single interval (use only ival2)
    Msg.Head.IVal1.Sec     := 0;
    Msg.Head.IVal1.uSec    := 0;
    Msg.Head.IVal2.Sec     := 0;
    Msg.Head.IVal2.uSec    := 250000; // 250 ms interval between CAN messages
    Msg.Frame[0].ID.Value  := $550; // not used if TX_CP_CAN_ID falg is set
    Msg.Frame[0].DLC       := 2;
    Msg.Frame[0].Data      := [$01,$02,$03,$04,$05,$06,$07,$08];
    Msg.Frame[1].ID.Value  := $551; // not used if TX_CP_CAN_ID falg is set
    Msg.Frame[1].DLC       := 4;
    Msg.Frame[1].Data      := [$11,$12,$13,$14,$15,$16,$17,$18];
    Msg.Frame[2].ID.Value  := $552; // not used if TX_CP_CAN_ID falg is set
    Msg.Frame[2].DLC       := 8;
    Msg.Frame[2].Data      := [$21,$22,$23,$24,$25,$26,$27,$28];

    if CanWrite(CanHandle, Msg, sizeof(Msg)) <> sizeof(Msg) then
    begin
      perror('Write');
      Exit;
    end;

    repeat
      WriteLn('Press any key to stop...');
      Msg.Frame[2].DataByte[0] := Msg.Frame[2].DataByte[0] + 1;
      for i := 0 to Msg.Frame[2].DLC - 1 do
        Write(Format('%02.2X ', [Msg.Frame[2].Data[i]]));
      WriteLn('');
      WriteLn('');
      CanWrite(CanHandle, Msg, sizeof(Msg)); // comment out this line if you do not want to publish changed frame[2]
      Delay(2000); // loop every 2 seconds
    until KeyPressed;

    if CanClose(CanHandle) < 0 then
    begin
      perror('Close');
      Exit;
    end;

  except
    on e: Exception do
      WriteLn(e.Message);
  end;
end.

