//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This is a SocketCAN demo converted from C to Pascal by Zeljko Avramovic.    //
//  Original example has GNU General Public License v3.0 and can be found at    //
//  https://github.com/craigpeacock/CAN-Examples                                //
//                                                                              //
//  Virtual CAN adapter vcan0 is hard coded and you can bring it up like this:  //
//  sudo modprobe vcan                                                          //
//  sudo ip link add dev vcan0 type vcan                                        //
//  sudo ip link set vcan0 mtu 72            # needed for CAN FD                //
//  sudo ip link set vcan0 up                                                   //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

program hlcanfdtransmit; // higher level pascalized version of canfdtransmit demo

{$mode delphi} {$H+}

uses
  cthreads, sysutils, sockets, baseunix, can, can.hl, can.raw;

procedure perror(const S : string);
begin
  WriteLn(S, ', SocketError = ', SocketError);
end;

var
  Err:          integer;
  CanHandle:    TCanHandle;
  CanAddr:      TCanSockAddr;
  CanInterface: TCanIfreq;
  CanFDFrame:   TCanFDFrame;
begin
  try
    Writeln('CAN FD Sockets Transmit Demo');
    CanHandle := CanSocket(SOCK_RAW, CAN_RAW);
    if CanHandle < 0 then
    begin
      if CanHandle = ESysENOPROTOOPT then
        WriteLn('CAN_RAW_FD_FRAMES socket option is not supported. Your kernel is probably too old.');
      perror('Socket');
      Exit;
    end;

    CanInterface.Name := 'vcan0' + #0;
    if CanIOCtl(CanHandle, SIOCGIFINDEX, CanInterface) < 0 then
    begin
      perror('IOctl');
      Exit;
    end;

    CanAddr.Clear;
    CanAddr.CanFamily  := AF_CAN;
    CanAddr.CanIfIndex := CanInterface.IfIndex;

    if CanBind(CanHandle, CanAddr) < 0 then
    begin
      perror('Bind');
      Exit;
    end;

    CanFDFrame.ID.Value := $444444; // CAN FD message extended ID
    CanFDFrame.ID.EFF   := true;    // confirm that we have 29-bit CAN ID
    CanFDFrame.Len      := 20;      // payload length will be 20 bytes
    CanFDFrame.Data     := [$01, $02, $03, $04, $05, $06, $07, $08, $09, $0A, $0B, $0C, $0D, $0E, $0F, $10,
                            $11, $12, $13, $14, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
                            $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
                            $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00];
                            // shows as '0x444 [40] 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 13 14' in canfdreceive
    {
    CanFDFrame.Data[0]  := $01; // alternative for the above
    CanFDFrame.Data[1]  := $02; // shows as '0x444 [40] 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 13 14' in canfdreceive
    CanFDFrame.Data[2]  := $03;
    CanFDFrame.Data[3]  := $04;
    CanFDFrame.Data[4]  := $05;
    CanFDFrame.Data[5]  := $06;
    CanFDFrame.Data[6]  := $07;
    CanFDFrame.Data[7]  := $08;
    CanFDFrame.Data[8]  := $09;
    CanFDFrame.Data[9]  := $0A;
    CanFDFrame.Data[10] := $0B;
    CanFDFrame.Data[11] := $0C;
    CanFDFrame.Data[12] := $0D;
    CanFDFrame.Data[13] := $0E;
    CanFDFrame.Data[14] := $0F;
    CanFDFrame.Data[15] := $10;
    CanFDFrame.Data[16] := $11;
    CanFDFrame.Data[17] := $12;
    CanFDFrame.Data[18] := $13;
    CanFDFrame.Data[19] := $14;
    }

    Err := CanSetSockOpt(CanHandle, SOL_CAN_RAW, CAN_RAW_FD_FRAMES, ENABLE_CAN_OPTION, SizeOf(ENABLE_CAN_OPTION));
    if Err <> 0 then
    begin
      if Err = ESysENOPROTOOPT then
        WriteLn('CAN FD Protocol not available. Try newer kernel');
      perror('SetSockopt');
      Exit;
    end;

    if CanWrite(CanHandle, CanFDFrame) <> SizeOf(CanFDFrame) then
    begin
      perror('Write');
      Exit;
    end;

    if CanClose(CanHandle) < 0 then
    begin
      perror('Close');
      Exit;
    end;

  except
    on e: Exception do
      WriteLn(e.Message);
  end;
end.

