//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This is a SocketCAN demo converted from C to Pascal by Zeljko Avramovic.    //
//  Original example has GNU General Public License v3.0 and can be found at    //
//  https://github.com/craigpeacock/CAN-Examples                                //
//                                                                              //
//  Virtual CAN adapter vcan0 is hard coded and you can bring it up like this:  //
//  sudo modprobe vcan                                                          //
//  sudo ip link add dev vcan0 type vcan                                        //
//  sudo ip link set vcan0 mtu 72            # needed for CAN FD                //
//  sudo ip link set vcan0 up                                                   //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

program hlcanfilter; // higher level pascalized version of canfilter demo

{$mode delphi} {$H+}

uses
  cthreads, sysutils, sockets, can, can.hl, can.raw;

procedure perror(const S : string);
begin
  WriteLn(S, ', SocketError = ', SocketError);
end;

var
  i, BytesNo:   byte;
  CanHandle:    TCanHandle;
  CanAddr:      TCanSockAddr;
  CanInterface: TCanIfreq;
  CanFrame:     TCanFrame;
  CanFilter:    array[0..1] of TCanFilter;
begin
  try
    Writeln('CAN Sockets Filter Demo');
    CanHandle := CanSocket(SOCK_RAW, CAN_RAW);
    if CanHandle < 0 then
    begin
      perror('Socket');
      Exit;
    end;

    CanInterface.Name := 'vcan0' + #0;
    if CanIOCtl(CanHandle, SIOCGIFINDEX, CanInterface) < 0 then
    begin
      perror('IOctl');
      Exit;
    end;

    CanAddr.Clear;
    CanAddr.CanFamily  := AF_CAN;
    CanAddr.CanIfIndex := CanInterface.IfIndex;

    if CanBind(CanHandle, CanAddr) < 0 then
    begin
      perror('Bind');
      Exit;
    end;

    CanFilter[0].ID.Value := $550; // 1st CAN ID
    CanFilter[0].Mask     := $7F0; // 1st CAN MASK matches all frames with identifiers from $550 to $55F
    CanFilter[1].ID.Value := $200; // 2nd CAN ID
    CanFilter[1].Mask     := $7FF; // 2nd CAN MASK matches only frames with identifier equal to $200 and nothing else

    CanSetSockOpt(CanHandle, SOL_CAN_RAW, CAN_RAW_FILTER, CanFilter, SizeOf(CanFilter));

    BytesNo := CanRead(CanHandle, CanFrame);
    if BytesNo < 0 then
    begin
      perror('Read');
      Exit;
    end;
    if BytesNo < SizeOf(TCanFrame) then // paranoid check
    begin
      perror('Read: Incomplete CAN frame');
      Exit;
    end;

    Write(format('0x%03X [%d] ', [CanFrame.ID.Value, CanFrame.DLC]));
    for i := 0 to CanFrame.DLC - 1 do
      Write(Format('%02.2X ', [CanFrame.Data[i]]));
    WriteLn('');

    if CanClose(CanHandle) < 0 then
    begin
      perror('Close');
      Exit;
    end;

  except
    on e: Exception do
      WriteLn(e.Message);
  end;
end.

