//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This is a SocketCAN demo converted from C to Pascal by Zeljko Avramovic.    //
//  Original example has GNU General Public License v3.0 and can be found at    //
//  https://github.com/craigpeacock/CAN-Examples                                //
//                                                                              //
//  Virtual CAN adapter vcan0 is hard coded and you can bring it up like this:  //
//  sudo modprobe vcan                                                          //
//  sudo ip link add dev vcan0 type vcan                                        //
//  sudo ip link set vcan0 mtu 72            # needed for CAN FD                //
//  sudo ip link set vcan0 up                                                   //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

program hlcantransmit; // higher level pascalized version of cantransmit demo

{$mode delphi} {$H+}

uses
  cthreads, sysutils, sockets, can, can.hl;

procedure perror(const S : string);
begin
  WriteLn(S, ', SocketError = ', SocketError);
end;

var
  CanHandle:    TCanHandle;
  CanAddr:      TCanSockAddr;
  CanInterface: TCanIfreq;
  CanFrame:     TCanFrame;
begin
  try
    Writeln('CAN Sockets Transmit Demo');
    // CanHandle := CanSocket(PF_CAN, SOCK_RAW, CAN_RAW); // the same
    // CanHandle := CanSocket(pfCAN,  SOCK_RAW, CAN_RAW); // the same
    CanHandle    := CanSocket(SOCK_RAW, CAN_RAW);         // the same
    if CanHandle < 0 then
    begin
      perror('Socket');
      Exit;
    end;

    CanInterface.Name := 'vcan0' + #0;
    if CanIOCtl(CanHandle, SIOCGIFINDEX, CanInterface) < 0 then
    begin
      perror('IOctl');
      Exit;
    end;

    CanAddr.Clear;
    CanAddr.CanFamily  := AF_CAN;
    CanAddr.CanIfIndex := CanInterface.IfIndex;

    if CanBind(CanHandle, CanAddr) < 0 then
    begin
      perror('Bind');
      Exit;
    end;

    CanFrame.ID.Value := $555;    // CAN message ID
    CanFrame.DLC      := 5;       // payload length will be 5 bytes
    CanFrame.DataText := 'Hello'; // datatext field is a variant record of chars overlaying data field for easy string messages, shows as 0x555 [5] 48 65 6C 6C 6F
    // CanFrame.data     := [$11, $22, $33, $44, $55, $00, $00, $00]; // if you want to send binary data instead of text, shows as 0x555 [5] 11 22 33 44 55
    // CanFrame.data[0]  := $11; // alternative for the above, shows as 0x555 [5] 11 22 33 44 55 in canreceive
    // CanFrame.data[1]  := $22;
    // CanFrame.data[2]  := $33;
    // CanFrame.data[3]  := $44;
    // CanFrame.data[4]  := $55;
    // CanFrame.DataQword[0] := $1122334455667788; // quite different from the above and shows in reverse order as 0x555 [5] 88 77 66 55 44 in canreceive

    if CanWrite(CanHandle, CanFrame) <> SizeOf(CanFrame) then
    begin
      perror('Write');
      Exit;
    end;

    if CanClose(CanHandle) < 0 then
    begin
      perror('Close');
      Exit;
    end;

  except
    on e: Exception do
      WriteLn(e.Message);
  end;
end.

